"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImageProcessor = void 0;
const os_1 = __importDefault(require("os"));
const path_1 = __importDefault(require("path"));
const crypto_1 = __importDefault(require("crypto"));
const promises_1 = require("fs/promises");
const resizer_1 = require("./resizer");
/**
 * sharp-mozjpeg-image-processor
 * main class
 */
class ImageProcessor {
    /**
     * @param tmpDir temporary directory
     * @param log Logging function
     */
    constructor(tmpDir, log) {
        this.tmpRootDir = tmpDir ?? os_1.default.tmpdir();
        this.log = log ?? ((ctx, msg) => {
            console.log(msg, JSON.stringify(ctx, null, 2));
        });
    }
    /**
     * execute generating image files from source file stream
     *
     * @param fileStream source image file stream
     * @param imageInfos generating image file infos
     * @param quality jpeg quality 0~100
     * @returns generated image file infos
     */
    async execute(fileStream, imageInfos, quality = 80) {
        const tmpDir = path_1.default.join(this.tmpRootDir, 'work' + crypto_1.default.randomBytes(8).readUInt32LE(0));
        await (0, promises_1.mkdir)(tmpDir);
        if (imageInfos.length === 0) {
            throw new Error('imageInfo requires at least 1 item');
        }
        const sortedImgInfos = imageInfos.sort((a, b) => (b.width * b.height - a.width * a.height));
        const maxImgInfo = sortedImgInfos.shift();
        const maxImgFilePath = path_1.default.join(tmpDir, `${maxImgInfo.kind}.jpg`);
        const maxOutImgInfo = await this.generateArrangedFile(fileStream, maxImgFilePath, maxImgInfo);
        const results = [{
                kind: maxOutImgInfo.kind,
                format: 'jpeg',
                width: maxOutImgInfo.width,
                height: maxOutImgInfo.height,
                filePath: maxImgFilePath,
            }];
        // generating scaled and optimized images
        for (const imgInfo of sortedImgInfos) {
            const destFilePath = path_1.default.join(tmpDir, `${imgInfo.kind}.jpg`);
            const outImgInfo = await this.generateScaledFile(maxImgFilePath, destFilePath, imgInfo);
            results.push({
                kind: outImgInfo.kind,
                format: 'jpeg',
                width: outImgInfo.width,
                height: outImgInfo.height,
                filePath: destFilePath,
            });
        }
        return results;
    }
    // convert file stream to maximum rotation arraged file
    async generateArrangedFile(fileStream, destFilePath, imgInfo) {
        const outImgInfo = await (0, resizer_1.convertStreamToArrangedFile)(fileStream, destFilePath, imgInfo.width, imgInfo.height);
        const result = { ...outImgInfo, kind: imgInfo.kind, filePath: destFilePath };
        this.log(result, 'Generated maximum scaled image file');
        return result;
    }
    async generateScaledFile(srcFilePath, destFilePath, imgInfo) {
        const outImgInfo = await (0, resizer_1.convertToScaledFile)(srcFilePath, destFilePath, imgInfo.crop ?? false, imgInfo.width, imgInfo.height);
        const result = { ...outImgInfo, kind: imgInfo.kind, filePath: destFilePath };
        this.log(result, 'Generated scaled image file from maximum scaled one');
        return result;
    }
}
exports.ImageProcessor = ImageProcessor;
